{
 DOSamp Extender - The Nullsoft's DOSamp extension utility - configuration.
 Copyright (C) 1998-2003  Henrich Fukna <fuky@azet.sk>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}
program IniDosAmpExtender;

{$M 16384, 0, 655360}

uses Dos,
     App, Dialogs, Drivers, MsgBox, Objects, StdDlg, Views,
     Lib;

const
                                         { event commands }
  cmOptions  = 100;                      { configuration }
  cmPlayList = 101;                      { playlist editor }
  cmAddSong  = 110;                      { add song }
  cmAddSongs = 111;                      { add songs }
  cmDelSong  = 112;                      { remove song }
  cmSavePls  = 113;                      { save playlist }
  cmLoadPls  = 114;                      { load playlist }

{$I DOSAMPEX.INC }

type
  PPlayListCollection = ^TPlayListCollection; { Playlist collection }
  TPlayListCollection = object(TStringCollection)
    wasError: Boolean;                   { handle error flag }
    constructor Init(ALimit, ADelta: Integer); { Initialize playlist collection }
    procedure Error(Code, Info: Integer); virtual; { Handle error(s) }
  end;

  TPlayListRec = record                  { Playlist item }
    Coll: PPlayListCollection;           { playlist }
    Indx: Word;                          { selected item }
    Name: PString;                       { playlist file-name }
  end;

  TConfigRec = record                    { Configuration record }
    DosAmpSet: Word;                     { external player settings }
    DosAmpExSet: Word;                   { dosampex settings }
  end;

  TIniDax = object(TApplication)         { Application }
    constructor Init(aMode: String);     { Initialize application }
    procedure HandleEvent(var Event: TEvent); virtual;
                                         { Handle events }
    procedure InitDeskTop; virtual;      { Initialize desktop }
    procedure InitMenuBar; virtual;      { Initialize menu }
    procedure InitStatusLine; virtual;   { Initialize status }
  end;

  PConfigDlg = ^TConfigDlg;              { Application config dialog }
  TConfigDlg = object(TDialog)
    Rec: TConfigRec;                     { Configuration record }
    constructor Init;                    { Initialize config dialog }
    procedure HandleEvent(var Event: TEvent); virtual;
                                         { Handle events }
  end;

  PPlayListDlg = ^TPlayListDlg;          { Application playlist dialog }
  TPlayListDlg = object(TDialog)
    Rec: TPlayListRec;
    PlsIni: PathStr;                     { PLS playlist file }
    Modified: Boolean;                   { modification flag }
    constructor Init;                    { Initialize playlist dialog }
    procedure HandleEvent(var Event: TEvent); virtual;
                                         { Handle events }
  private
    procedure LoadPlayList;              { Load playlist and set dialog data }
    procedure SavePlayList;              { Store playlist and set dialog data }
  end;

  PGetDirDlg = ^TGetDirDlg;              { Application change directory dialog }
  TGetDirDlg = object(TChDirDialog)
    procedure GetData(var Rec); virtual; { Get dialog data }
    function Valid(Command: Word): Boolean; virtual;
                                         { Validate dialog data }
  end;

{********** Application ******************************************}

{*
 * Initialize config dialog.
 *
 * @param aMode application mode
 *}
constructor TIniDax.Init;
var
  Event: TEvent;
begin
  inherited Init;                        { call parrent }

  if aMode = DaxIniSwitch_Cfg then
    Event.Command:= cmOptions            { requested configuration }
  else
  if aMode = DaxIniSwitch_PlayList then
    Event.Command:= cmPlayList           { requested playlist editor }
  else
    Event.Command:= cmQuit;              { otherwise quit command }

  Event.What:= evCommand;                { event is command }
  PutEvent(Event);                       { send event }
end;

{*
 * Handle events.
 *
 * @param Event application events
 *}
procedure TIniDax.HandleEvent;

  {*
   * Show configuration.
   *}
  procedure Configuration;
  var
    Dlg: PDialog;
  begin
    Dlg:= New(PConfigDlg, Init);         { initialize app. config dialog }
    Desktop^.ExecView(Dlg);              { execute dialog }
    Dispose(Dlg, Done);                  { destroy dialog }
  end;

  {*
   * Show playlist editor.
   *}
  procedure PlayListEditor;
  var
    Dlg: PDialog;
  begin
    Dlg:= New(PPlayListDlg, Init);       { initialize app. playlist dialog }
    DeskTop^.ExecView(Dlg);              { execute dialog }
    Dispose(Dlg, Done);                  { destroy dialog }
  end;

begin
  inherited HandleEvent(Event);          { call parrent }
  if Event.What = evCommand then begin
    case Event.Command of
      cmOptions:  Configuration;         { show configuration }
      cmPlayList: PlayListEditor;        { show playlist }
    else
      Exit;                              { otherwise exit handler }
    end;
    ClearEvent(Event);                   { event handled - clear it }
    Event.What:= evCommand;              { event is command }
    Event.Command:= cmQuit;              { quit command }
    PutEvent(Event);                     { send event }
  end;
end;

{*
 * Initialize desktop.
 *}
procedure TIniDax.InitDeskTop;
var
  Rect: TRect;
begin
  GetExtent(Rect);                       { get fullscreen coordinates }
  Desktop:= New(PDesktop, Init(Rect));   { create desktop }
end;

{*
 * Initialize menu.
 *}
procedure TIniDax.InitMenuBar;
begin                                    { menu not used }
end;

{*
 * Initialize status.
 *}
procedure TIniDax.InitStatusLine;
begin                                    { status not used }
end;

{********** Application config dialog ****************************}

{*
 * Initialize config dialog.
 *}
constructor TConfigDlg.Init;
var
  I: Integer;
  S, T: String;
  R: TRect;
begin
  R.Assign(0, 0, 40, 17);
  inherited Init(R, 'Configuration');    { register window }

  Options:= Options or ofCentered;       { setup window }
  R.Assign(3, 2, 37, 3);
  Insert(New(PStaticText, Init(R, 'DosAMP setting:')));
  R.Assign(3, 3, 37, 7);
  Insert(New(PCheckBoxes, Init(R,
    NewSItem('~M~ono channel (downmix)',
    NewSItem('~H~alf samplerate (downsample)',
    NewSItem('~8~-bit per sample',
    Newsitem('~D~MA size 32768',
    nil))))
  )));
  R.Assign(3, 8, 37, 9);
  Insert(New(PStaticText, Init(R, 'DosAMP extender setting:')));
  R.Assign(3, 9, 37, 12);
  Insert(New(PCheckBoxes, Init(R,
    NewSItem('~R~epeatly playing',
    NewSItem('Re~v~erse order',
    NewSItem('~S~huffle (random) order',
    nil)))
  )));
  R.Assign(2, 14, 14, 16);
  Insert(New(PButton, Init(R, '~P~laylist', cmPlayList, bfNormal)));
  R.Assign(14, 14, 26, 16);
  Insert(New(PButton, Init(R, 'O~K~', cmOk, bfDefault)));
  R.Assign(26, 14, 38, 16);
  Insert(New(PButton, Init(R, 'Cancel', cmCancel, bfNormal)));
  SelectNext(False);

  with Rec do begin                      { setup data }
    DosAmpSet:= 0;                       { external player settings }
                                         { get options for external player }
    GetPrivateProfileString(DaxIniSection, DaxIniKey_DosAmp, '',
      S, 80, DaxIni);
    for I:= 0 to GetParseCount(S, ' ')-1 do begin { for all options }
      T:= GetParseString(I, S, ' ');     { get option }
      if T = DosAmpOption_DownMix then   { downmix option }
        DosAmpSet:= DosAmpSet or 1
      else
      if T = DosAmpOption_DownSample then{ downsample option}
        DosAmpSet:= DosAmpSet or 2
      else
      if T = DosAmpOption_8 then         { 8-bit option }
        DosAmpSet:= DosAmpSet or 4
      else
      if T = DosAmpOption_DmaSize then   { dma size option }
        DosAmpSet:= DosAmpSet or 8;
    end;

    DosAmpExSet:= 0;                     { dosampex settings }
                                         { get extended options }
    GetPrivateProfileString(DaxIniSection, DaxIniKey_DosAmpEx, '',
      S, 80, DaxIni);
    for I:= 0 to GetParseCount(S, ' ')-1 do begin { for all options }
      T:= GetParseString(I, S, ' ');     { get option }
      if T = DaxIniOption_Repeat then    { repeat option }
        DosAmpExSet:= DosAmpExSet or 1
      else
      if T = DaxIniOption_Reverse then   { reverse option }
        DosAmpExSet:= DosAmpExSet or 2
      else
      if T = DaxIniOption_Shuffle then   { shuffle option }
        DosAmpExSet:= DosAmpExSet or 4;
    end;
  end;
  SetData(Rec);                          { set dialog data }
end;

{*
 * Handle events.
 *
 * @param Event application events
 *}
procedure TConfigDlg.HandleEvent;

  {*
   * Show playlist.
   *}
  procedure PlayList;
  var
    Dlg: PDialog;
  begin
    Dlg:= New(PPlayListDlg, Init);       { initialize app. playlist dialog }
    DeskTop^.ExecView(Dlg);              { execute dialog }
    Dispose(Dlg, Done);                  { destroy dialog }
  end;

  {*
   * Store configuration setting.
   *}
  procedure Ok;
  var
    S: String;
  begin
    GetData(Rec);                        { get dialog data }
    with Rec do begin
      S:= '';                            { external player settings }
      if (DosAmpSet and 1) <> 0 then     { downmix option }
        S:= Concat(S, ' ', DosAmpOption_DownMix);
      if (DosAmpSet and 2) <> 0 then     { downsample option}
        S:= Concat(S, ' ', DosAmpOption_DownSample);
      if (DosAmpSet and 4) <> 0 then     { 8-bit option }
        S:= Concat(S, ' ', DosAmpOption_8);
      if (DosAmpSet and 8) <> 0 then     { dma size option }
        S:= Concat(S, ' ', DosAmpOption_DmaSize, ' 32768');
                                         { store options for external player }
      WritePrivateProfileString(DaxIniSection, DaxIniKey_DosAmp, S, DaxIni);

      S:= '';                            { dosampex settings }
      if (DosAmpExSet and 1) <> 0 then   { repeat option }
        S:= Concat(S, ' ', DaxIniOption_Repeat);
      if (DosAmpExSet and 2) <> 0 then   { reverse option }
        S:= Concat(S, ' ', DaxIniOption_Reverse);
      if (DosAmpExSet and 4) <> 0 then   { shuffle option }
        S:= Concat(S, ' ', DaxIniOption_Shuffle);
                                         { store extended options }
      WritePrivateProfileString(DaxIniSection, DaxIniKey_DosAmpEx, S, DaxIni);
    end;
  end;

begin
  if Event.What = evCommand then begin
    if Event.Command = cmPlaylist then
      PlayList                           { show playlist }
    else
    if Event.Command = cmOk then begin
      Ok;                                { save setting }
      inherited HandleEvent(Event);      { call parrent }
    end
    else
      inherited HandleEvent(Event);      { not handled - take chance parrent }

    ClearEvent(Event);                   { event handled - clear it }
  end
  else
    inherited HandleEvent(event);        { not handled - take chance parrent }
end;

{**********************************************************************}

constructor TPlayListDlg.Init;
var
  S: String;
  R: TRect;
  Control: PScrollBar;
begin
  R.Assign(0, 0, 52, 19);
  inherited Init(R, 'Playlist');         { register window }

  Options:= Options or ofCentered;       { setup window }
  R.Assign(49, 2, 50, 13);
  Control:= New(PScrollBar, Init(R));
  Insert(Control);
  R.Assign(3, 2, 49, 13);
  Insert(New(PListBox, Init(R, 1, Control)));
  R.Assign(3, 1, 49, 2);
  Insert(New(PParamText, Init(R, '%s', 1)));

  R.Assign(2, 14, 14, 16);
  Insert(New(PButton, Init(R, '~L~oad pls', cmLoadPls, bfNormal)));
  R.Assign(14, 14, 26, 16);
  Insert(New(PButton, Init(R, '~S~ave pls', cmSavePls, bfNormal)));
  R.Assign(2, 16, 14, 18);
  Insert(New(PButton, Init(R, 'Add ~f~ile', cmAddSong, bfNormal)));
  R.Assign(14, 16, 26, 18);
  Insert(New(PButton, Init(R, 'Add di~r~', cmAddSongs, bfNormal)));
  R.Assign(26, 16, 38, 18);
  Insert(New(PButton, Init(R, '~D~elete', cmDelSong, bfNormal)));
  R.Assign(26, 14, 38, 16);
  Insert(New(PButton, Init(R, 'Sele~c~t', cmOk, bfDefault)));
  R.Assign(38, 14, 50, 16);
  Insert(New(PButton, Init(R, 'Cancel', cmCancel, bfNormal)));
  SelectNext(False);
                                         { setup data }
  GetPrivateProfileString(DaxIniSection, DaxIniKey_PlayList,
    '', S, 80, DaxIni);
  PlsIni:= S;
  if PlsIni = '' then
    PlsIni:= DaxIni_PlsNone;             { set playlist title }

  LoadPlayList;                          { load playlist }
  SetData(Rec);                          { set dialog data }
end;

procedure TPlayListDlg.HandleEvent;

  {*
   * Add song to playlist.
   *}
  procedure AddSong;
  var
    I: Integer;
    S: PathStr;
    Dlg: PDialog;
    Coll: PPlayListCollection;
  begin
    Dlg:= New(PFileDialog, Init('*.MP3', 'Add a File', '~N~ame',
      fdOpenButton, 0));

    if DeskTop^.ExecView(Dlg) = cmFileOpen then begin
      Dlg^.GetData(S);                   { get new filename }
      GetData(Rec);                      { get dialog data }

                                         { copy list's items }
      Coll:= New(PPlayListCollection, Init(5, 5));
      for I:= 0 to Rec.Coll^.Count-1 do begin
        Coll^.Insert(NewStr(String(Rec.Coll^.At(I)^)));
        if Coll^.wasError then Break;
      end;
      if not(Coll^.wasError) then
        Coll^.Insert(NewStr(S));         { insert new filename }

      Coll^.wasError:= False;            { for dialog data we report }
      Rec.Coll:= Coll;                   { all ok }

      Modified:= True;                   { set modify flag }
      SetData(Rec);                      { set dialog data }
    end;
  end;

  {*
   * Remove song from playlist.
   *}
  procedure RemoveSong;
  var
    I: Integer;
    Coll: PPlayListCollection;
  begin
    GetData(Rec);                        { get dialog data }
    if Rec.Coll^.Count > 0 then begin    { is there to remove }
      Rec.Coll^.AtDelete(Rec.Indx);
                                         { copy list's items }
      Coll:= New(PPlayListCollection, Init(5, 5));
      for I:= 0 to Rec.Coll^.Count-1 do begin
        Coll^.Insert(NewStr(String(Rec.Coll^.At(I)^)));
        if Coll^.wasError then Break;
      end;

      Coll^.wasError:= False;            { for dialog data we report }
      Rec.Coll:= Coll;                   { all ok }
      if Coll^.Count = 0 then            { no data - select first }
        Rec.Indx:= 0
      else
      if Coll^.Count <= Rec.Indx then    { gr than data - select last }
        Rec.Indx:= Coll^.Count-1;

      Modified:= True;                   { set modify flag }
      SetData(Rec);                      { set dialog data }
    end;
  end;

  {*
   * Add recursively all specified songs to playlist.
   *}
  procedure AddSongs;

    procedure RecursiveDir(var aDir: String; var aMask: String;
      var aRec: Boolean; aColl: PPlayListCollection);
    var
      Path: String;
      Search: SearchRec;
    begin
      if Length(aDir) > 0 then begin
        if Length(aDir) > SizeOf(DirStr) then
          Exit;                          { direcotry length test }
        if aDir[Length(aDir)] <> '\' then
          aDir:= Concat(aDir, '\');      { add backslash, if needed }
      end;
      Path:= ConCat(aDir, '*.*');        { collect search mask }
      FindFirst(Path, AnyFile, Search);  { search files }
      while DosError = 0 do begin
                                         { witout current and parrent directory }
        if (Search.Name <> '.') and (Search.Name <> '..') then
          if ((Search.Attr and Directory) <> 0) then begin
                                         { for all directories }
            if aRec then begin           { if recursion enabled }
                                         { append deeper level }
              Path:= Concat(aDir, Search.Name, '\');
              RecursiveDir(Path, aMask, aRec, aColl);
            end;
          end else begin                 { for all files }
                                         { check with user's mask }
            Path:= Concat(aDir, Search.Name);
            if CmpName(aMask, Search.Name) then begin
              aColl^.Insert(NewStr(Path));
              if aColl^.wasError then Exit;
            end;
          end;
        if aColl^.wasError then Exit;    { collection integrity test }
        FindNext(Search);                { search next }
      end;
    end;

  var
    I: Integer;
    Recurse: Boolean;
    Dir: String;
    Mask: String;
    Dlg: PDialog;
    Coll: PPlayListCollection;

  begin
    Dlg:= New(PGetDirDlg, Init(cdNormal, 1));
    if DeskTop^.ExecView(Dlg) = cmOk then begin
      Dlg^.GetData(Dir);                 { get directory }
      Mask:= '*.MP3';                    { get filename wildcard mask }
      if InputBox('Confirm', 'Set files ~m~ask', Mask, 12) = cmOk then begin
        Recurse:= MessageBox(^M^C'Include subdirectories ?',
          nil, mfConfirmation + mfYesButton + mfNoButton) = cmYes;
                                         { get recursive flag }
        GetData(Rec);                    { get dialog data }
                                         { copy list's items }
        Coll:= New(PPlayListCollection, Init(5, 5));
        for I:= 0 to Rec.Coll^.Count-1 do begin
          Coll^.Insert(NewStr(String(Rec.Coll^.At(I)^)));
          if Coll^.wasError then Break;
        end;
        if not(Coll^.wasError) then      { fill list }
          RecursiveDir(Dir, Mask, Recurse, Coll);

        Coll^.wasError:= False;          { for dialog data we report }
        Rec.Coll:= Coll;                 { all ok }

        Modified:= True;                 { set modify flag }
        SetData(Rec);                    { set dialog data }
      end;
    end;
  end;

  {*
   * Load playlist.
   *}
  procedure LoadPls;
  var
    Dlg: PDialog;
  begin
    if Modified then                     { if current playlist modified }
                                         { ask for save }
      if MessageBox(^C'Playlist modified !'^M^C'Do you want save changes ?',
           nil, mfWarning + mfYesButton + mfNoButton) = cmYes then
      begin
        GetData(Rec);                    { get dialog data }
        SavePlayList;                    { save playlist }
      end;
                                         { open new playlist }
    Dlg:= New(PFileDialog, Init('*.PLS', 'Load a Playlist', '~N~ame',
      fdOpenButton, 0));

    if DeskTop^.ExecView(Dlg) = cmFileOpen then begin
      Dlg^.GetData(PlsIni);              { get new playlist filename }
      LoadPlayList;                      { load playlist }
      SetData(Rec);                      { set dialog data }
    end;
  end;

  {*
   * Save playlist.
   *}
  procedure SavePls;
  var
    I: LongInt;
    Dlg: PDialog;
    Coll: PPlayListCollection;
  begin
                                         { save current playlist }
    Dlg:= New(PFileDialog, Init('*.PLS', 'Save Playlist As', '~N~ame',
      fdOkButton, 0));

    if DeskTop^.ExecView(Dlg) = cmFileOpen then begin
      Dlg^.GetData(PlsIni);              { get new playlist filename }
      GetData(Rec);                      { get dialog data }
      SavePlayList;                      { save playlist }

                                         { copy list's items }
      Coll:= New(PPlayListCollection, Init(5, 5));
      for I:= 0 to Rec.Coll^.Count-1 do begin
        Coll^.Insert(NewStr(String(Rec.Coll^.At(I)^)));
        if Coll^.wasError then Break;
      end;

      Coll^.wasError:= False;            { for dialog data we report }
      Rec.Coll:= Coll;                   { all ok }
      Rec.Name:= @PlsIni;
      SetData(Rec);                      { set dialog data }
    end;
  end;

  {*
   * Close playlist dialog.
   *}
  procedure Ok;
  begin
    if Modified then
      if MessageBox(^C'Playlist modified !'^M^C'Do you want save changes ?',
           nil, mfWarning + mfYesButton + mfNoButton) = cmYes then
      begin
        if PlsIni = DaxIni_PlsNone then begin
          SavePls;                       { save playlist with }
                                         { filename question }
        end else begin
          GetData(Rec);                  { get dialog data }
          SavePlayList;                  { save playlist }
        end;
      end;
                                         { store current playlist filename }
    WritePrivateProfileString(DaxIniSection, DaxIniKey_PlayList,
      PlsIni, DaxIni);
  end;

begin
  case Event.What of
    evKeyDown:
      case Event.KeyCode of
        kbIns: AddSong;                  { add song to playlist }
        kbDel: RemoveSong;               { remove song from playlist }
      else
        inherited HandleEvent(Event);    { not handled - take chance parrent }
      end;

    evCommand:
      case Event.Command of
        cmAddSong: AddSong;              { add song to playlist }
        cmDelSong: RemoveSong;           { remove song from playlist }
        cmAddSongs: AddSongs;            { add songs to playlist }
        cmLoadPls: LoadPls;              { load playlist }
        cmSavePls: SavePls;              { save playlist }
        cmOk:
          begin
            Ok;                          { close playlist dialog }
            inherited HandleEvent(Event);{ call parrent }
          end;
      else
        inherited HandleEvent(Event);    { not handled - take chance parrent }
      end;
  else
    inherited HandleEvent(Event);        { not handled - take chance parrent }
  end;

  ClearEvent(Event);                     { event handled - clear it }
end;

{*
 * Load playlist and set dialog data.
 *}
procedure TPlayListDlg.LoadPlayList;
var
  I, Cnt: LongInt;
  S: String;
begin
  with Rec do begin                      { initialize record }
    Coll:= New(PPlayListCollection, Init(5, 5));
    Indx:= 0;
    Name:= @PlsIni;

    if PlsIni <> DaxIni_PlsNone then begin
      Cnt:= GetPrivateProfileInt(PlsIniSection, PlsIniKey_ListNum,
        0, PlsIni);                      { get number of entries }

      for I:= 1 to Cnt do begin
        Str(I, S);
        S:= PlsIniKey_ListItem + S;      { get item }
        GetPrivateProfileString(PlsIniSection, S, '', S, 80, PlsIni);

        if S <> '' then begin
          S:= StrUpr(S);                 { convert to upper-case }
          Coll^.Insert(NewStr(S));       { insert to collection }
          if Coll^.wasError then Break;
        end;
      end;
    end;
  end;
  Modified:= False;                      { clear modify flag }
end;

{*
 * Store playlist and set dialog data.
 *}
procedure TPlayListDlg.SavePlayList;
var
  I: LongInt;
  S: String;
  F: File;
begin
  Assign(F, PlsIni);
  {$I-}
  Erase(F);                              { erase old file (if exists) }
  {$I+}
  if ioResult <> 0 then ;

  for I:= 1 to Rec.Coll^.Count do begin  { create new file }
    Str(I, S);
    S:= PlsIniKey_ListItem + S;          { item }

    WritePrivateProfileString(PlsIniSection, S,
      String(Rec.Coll^.At(I-1)^), PlsIni);
  end;
  Str(Rec.Coll^.Count, S);               { number of entries }
  WritePrivateProfileString(PlsIniSection, PlsIniKey_ListNum, S, PlsIni);

  Modified:= False;                      { clear modify flag }
end;

{********** Application change directory dialog ******************}

{*
 * Get dialog data.
 *
 * @param Rec points to dialog data
 *}
procedure TGetDirDlg.GetData;
begin
  DirInput^.GetData(Rec);                { fill dialog data }
end;

{*
 * Validate dialog data.
 *
 * @param Command application event command
 * @return true if dialog data is valid, otherwise false
 *}
function TGetDirDlg.Valid;
var
  B: Boolean;
  Dir: PathStr;
begin
  GetDir(0, Dir);                        { store current directory }
  B:= inherited Valid(Command);          { call parrent validate proc. }
  ChDir(Dir);                            { restore directory }
  Valid:= B;                             { retrun status }
end;

{********** PlayList collection **********************************}

{*
 * Initialize playlist collection.
 *
 * @param ALimit initial size
 * @param ADelta grow increment
 *}
constructor TPlayListCollection.Init(ALimit, ADelta: Integer);
begin
  inherited Init(ALimit, ADelta);        { call parrent }
  wasError:= False;                      { initialize error flag }
end;

{*
 * Handle collection errors.
 *
 * @param Code error code (coXXXX constants)
 * @param Info depend aditional info
 *}
procedure TPlayListCollection.Error(Code, Info: Integer);
begin
  wasError:= True;                       { set error flag }
end;

{********** main module ******************************************}

var
  I: Integer;
  S: String;
  IniDax: TIniDax;

begin
  S:= '';                                { set full path to configuration }
  for I:= 0 to GetParseCount(ParamStr(0), '\')-2 do begin
    S:= Concat(S, GetParseString(I, ParamStr(0), '\'), '\');
  end;
  DaxIni:= Concat(S, DaxIni);

  if (ParamCount <> 1) or                { if not called from dosampex }
     ((ParamStr(1) <> DaxIniSwitch_Cfg) and
      (ParamStr(1) <> DaxIniSwitch_PlayList)) then
  begin
    PrintStr('This program is invoked and required by DOSAMPEX' + #13#10);
    PrintStr('Type: `DOSAMPEX -h'' for help.' + #13#10);
    Halt(255);
  end;

  Inidax.Init(ParamStr(1));              { initialize application }
  IniDax.Run;                            { run application }
  IniDax.Done;                           { destroy application }
end.